%% LOAD DATA

% The EKF and UKF implementations were adapted from the work of Prof. Dan Simon

% Author: Andrea Tuveri
clear
clc
%% CgSSL3_glucose 20g/L - extraction of data from Excel

% The following are all - Experimental data -

% Whole set of data from Numera
% (this are the data collected with the autosampler and analyzed manually;
% they are off-line data)

if exist('num','var')
    num = num;
else
    filename = 'CDW_2nd_ATU.xlsx';
    num = xlsread(filename);
end

% Whole set of data from Lucullus
%(this data are the data collected with the different measurement devices
% connected to the system; they give you at line measurements)

if exist('num2','var')
    num2 = num2;
else
    filename2 = 'CDW_ATU_18042020.xlsx';
    num2 = xlsread(filename2);  
end

%sugars
if exist('num3','var')
    num3 = num3;
else
    filename3 = 'sugar_CDW.xlsx';
    num3 = xlsread(filename3);  
end


% Save the different collected data
save('num')
save('num2')
save('num3')

%% Data Extraction for Matlab

% Extracing data from Numera to Matlab
number = 1; %step for data extraction
start = 1;  %start
slutt = 13; %end
t_CDW    = num(start:number:slutt,15);        % [h]
CDW   = num(start:number:slutt,16);   % Biomass [g/L]
std_CDW    = num(start:number:slutt,17);        % [g/L]
OD_600    = num(start:number:slutt,18);        % [g/L]

% Extracing Sugars
number = 1; %step for data extraction
start = 3;  %start
slutt = 17; %end
t_S    = num3(start:number:slutt,2);        % [h]
Sugar   = num3(start:number:slutt,6);   % Biomass [g/L]

% Extracing data from Lucullus to Matlab
start = 1;
fine = 16886;
slutt = 1;
t_luc = num2(start:slutt:fine,1);
Pump_Acid= num2(start:slutt:fine,35);
Pump_Base= num2(start:slutt:fine,36);
Air_flow= num2(start:slutt:fine,39);
CO2_Conc= num2(start:slutt:fine,41);
O2_Conc= num2(start:slutt:fine,42);
Feed_luc = num2(start:slutt:fine,43);
GM_flow= num2(start:slutt:fine,50);
NIR = num2(start:slutt:fine,48).*0.3634;
pH= num2(start:slutt:fine,49);
pO2= num2(start:slutt:fine,50);
Stirrer= num2(start:slutt:fine,51);
Temperature= num2(start:slutt:fine,52);

Glucosio = 4.501965589*CO2_Conc;

%% Glucose measurements

%Correct OD_exp
gluc_exp_corr_Array = [];

for i = 1:length(t_luc)
    gluc_exp_corr = -366.93*NIR(i)^2 + 9.3808*NIR(i) + 19.907 + Feed_luc(i)*0.00210*360;

    gluc_exp_corr_Array = [gluc_exp_corr_Array gluc_exp_corr];
end

GLUC = gluc_exp_corr_Array';

%% NIR correction

%Correct OD_exp
OD_exp_corr_Array = [];
delta = 1;
for i = 1:length(t_luc)
    OD_exp_corr = 110.3*NIR(i)^2 + -3.254*NIR(i) + 1.878;

    OD_exp_corr_Array = [OD_exp_corr_Array OD_exp_corr];
end

NIR = OD_exp_corr_Array';

%% OD Correction
       
OD_600_corr_Array = [];

for k = 1:length(t_CDW)
    
    OD_600_corr = 0.3869*OD_600(k) + 1.191;

    OD_600_corr_Array = [OD_600_corr_Array OD_600_corr];
end

OD_600 = OD_600_corr_Array';

%% CO2 correction

% Adjusting CO2 concentration from V% to g/L
CO2_density = 1.8; % g/L
CO2inAir_volpercent = 0.0282;

for i = 1:length(CO2_Conc)
    Volume_CO2 = (Air_flow(i).*((CO2_Conc(i)-CO2inAir_volpercent)))/100; %L CO2/min
    CO2_excreted(i,1) = (Volume_CO2*CO2_density)*60; % g CO2/hr
end

CO2_conc_excreted = CO2_excreted ./(Air_flow .*60); %g CO2/L
CO2_flow_excreted = gradient(CO2_conc_excreted) ./gradient(t_luc); %g CO2/(L *h)

Q_CO2 = cumtrapz(t_luc,CO2_excreted); %[g]

%% calculate volume

global tspan feed

tspan = t_luc;
feed = Feed_luc;

[Time,Vol] = Volume_calc();
CO2_Conc_actual = Q_CO2./(Vol);

%% Glucose measurements

%Correct OD_exp
gluc_CO2_exp_corr_Array = [];

for i = 1:length(t_luc)
    gluc_CO2_exp_corr = -0.0342*Q_CO2(i)^2 - 0.4566*Q_CO2(i) + 20.274;
    gluc_CO2_exp_corr_Array = [gluc_CO2_exp_corr_Array gluc_CO2_exp_corr];
end

GLUC_CO2 = gluc_CO2_exp_corr_Array';

%% Correlation Curve CO2

%Correct OD_exp
CDW_exp_corr_Array = [];
delta = 1;
for i = 1:length(t_luc)
    CDW_exp_corr = 0.759*CO2_Conc_actual(i) + 1.817;

    CDW_exp_corr_Array = [CDW_exp_corr_Array CDW_exp_corr];
end

CO2_Conc_Actual = CDW_exp_corr_Array';

%% Simulate your system

t0 = 0;
dt = 10/3600;
tend = 150;

rpmlist = Stirrer';
Feed = Feed_luc';
pH = pH';
CO2_conc = CO2_Conc';
Air_flow = Air_flow';
Temp = Temperature';

[sim,t_sim] = solver_CDW_2nd(t0,dt,tend,rpmlist,Feed,pH,CO2_conc,Air_flow,Temp);

%% decrease the vectors size 
%we want to reduce the "sampling" frequency of the data

start = 1;
fine = 16886;
slutt = 6;
time_luc = t_luc;
Feeding_luc = Feed_luc;
NIR_cov = NIR;
CO2_cov = CO2_Conc_Actual;
t_luc = t_luc(start:slutt:fine);
Pump_Acid= Pump_Acid(start:slutt:fine);
Pump_Base= Pump_Base(start:slutt:fine);
Air_flow= Air_flow(start:slutt:fine);
CO2_Conc= CO2_Conc(start:slutt:fine);
O2_Conc= O2_Conc(start:slutt:fine);
Feed_luc = Feed_luc(start:slutt:fine);
GM_flow= GM_flow(start:slutt:fine);
NIR = NIR(start:slutt:fine);
pH= pH(start:slutt:fine);
pO2= pO2(start:slutt:fine);
Stirrer= Stirrer(start:slutt:fine);
Temperature= Temperature(start:slutt:fine);
CO2_Conc_Actual = CO2_Conc_Actual(start:slutt:fine);
CO2_Conc_actual = CO2_Conc_actual(start:slutt:fine);
Q_CO2 = Q_CO2(start:slutt:fine);

%% UKF vs EKF

% Starting value for P
P = diag([1 1 1 1]); % error covariance matrix
Pukf = P;

%Initial Conditions
Vol = 1.5; %L
Bio = 1.2; %g/L
Sub = 20; %g/L
CO2 = 0; %g/L

% Parameter inputs
Ks = 0.0060;         % g L^-1 
Y_XS = 0.4719;
mu_max = 0.1529;   % h^-1
kd =  0.0074;
Sin = 100;         % g/L
Y_CO2X = 0.7880; %estimated

% Initial conditions
x = [Vol; Bio; Sub; CO2];

xhat = x;
xhatukf = xhat;
I = eye(length(xhat));

T = 1;
t0 = 0;

tend = length(t_luc);
dt = 0.001;
deltat = dt;

n = 4; %number of states
n2 = 2*n;

xArray = zeros(n,tend);
z1Array = zeros(1,tend);
z2Array = zeros(1,tend);
xhatArray = zeros(n,tend);
xhatukfArray = zeros(n,tend);
PArray = zeros(n,tend);
PukfArray = zeros(n,tend);
KukfArray = zeros(n,tend);
time_simulatedArray = zeros(1,tend);
x_simulatedArray = zeros(n,tend);
errArray = zeros(4,tend);
rankArray = zeros(1,tend);

W = ones(n2,1) / n2; % UKF weights

for t = T : T : tend
    tic;
    t
    time = t_luc(t)
% Model simulation
   for tau = dt : dt : T/60
       tid = t_luc(t) + tau;
       [~, idx] = min(abs(time_luc - tid));
       Fin = Feeding_luc(idx)*0.00210;
       if Fin <= 8.7576e-40
           Fin = 0;
       end
       xdot(1,1) = Fin;
       xdot(2,1) = -(x(2) ./x(1)) .*Fin + mu_max .*(x(3) ./(Ks + x(3))) .*x(2) - kd .*x(2); %Biomass
       xdot(3,1) = (Sin .*Fin) ./ x(1) - mu_max .*(x(3) ./(Ks + x(3))) .*x(2) ./Y_XS - (x(3) ./ x(1)) .*Fin; % Substrate
       xdot(4,1) = -(Fin./x(1)) .* x(4) + (mu_max .*(x(3) ./(Ks + x(3))) .*x(2))./Y_CO2X; %CO2
       x = x + xdot * dt;
   end
%measurement
    Q = diag([1 0.01*1.954 0.0006 0.00001]); % define the process noise
    z1 = 0.6655*NIR(t) + (1-0.6655)*CO2_Conc_Actual(t);
    z2 = CO2_Conc_actual(t);
    z3 = 4.501965589*CO2_Conc(t);
    z4 = x(1);
    if t_luc(t) >= 11.7
    H = [1 0 0 0; 0 1 0 0; 0 0 0 1; 0 0 1 0];
    R = diag([1 1.954 1 0.8]);
    else
    z3 = NaN;
    H = [1 0 0 0; 0 1 0 0; 0 0 0 1];
    R = diag([1 1.954 1]);
    end
    
% EKF continuous part  
   for tau = dt : dt : T/60
       tid = t_luc(t) + tau;
       [~, idx] = min(abs(time_luc - tid));
       Fin = Feeding_luc(idx)*0.00210;
       if Fin <= 8.7576e-40
           Fin = 0;
       end
       xhatdot(1,1) = Fin;
       xhatdot(2,1) = -(xhat(2) ./xhat(1)) .*Fin + mu_max .*(xhat(3) ./(Ks + xhat(3))) .*xhat(2) - kd .*xhat(2); %Biomass
       xhatdot(3,1) = (Sin .*Fin) ./ xhat(1) - mu_max .*(xhat(3) ./(Ks + xhat(3))) .*xhat(2) ./Y_XS - (xhat(3) ./ xhat(1)) .*Fin; % Substrate
       xhatdot(4,1) = -(Fin./xhat(1)) .* xhat(4) + (mu_max .*(xhat(3) ./(Ks + xhat(3))) .*xhat(2))./Y_CO2X; %CO2
       xhat = xhat + xhatdot * dt;
       V = xhat(1); X = xhat(2); S = xhat(3); CO2 = xhat(4);

       F = [                           0,                                0,                                                                   0,      0;
           (Fin*X)/V^2, (S*mu_max)/(Ks + S) - Fin/V - kd,                       (X*mu_max)/(Ks + S) - (S*X*mu_max)/(Ks + S)^2,      0;
           (Fin*S)/V^2 - (Fin*Sin)/V^2,      -(S*mu_max)/(Y_XS*(Ks + S)), (S*X*mu_max)/(Y_XS*(Ks + S)^2) - (X*mu_max)/(Y_XS*(Ks + S)) - Fin/V,      0;
           (CO2*Fin)/V^2,     (S*mu_max)/(Y_CO2X*(Ks + S)),     (X*mu_max)/(Y_CO2X*(Ks + S)) - (S*X*mu_max)/(Y_CO2X*(Ks + S)^2), -Fin/V];
 
                       
       Pdot = F * P + P * F' + Q * dt;
       P = P + Pdot * dt;
   end
   
   % EKF discrete part
   K = P * H' * pinv(H * P * H' + R);
   zhat = H*xhat;
   if t_luc(t) >= 11.7
   z_m = [z4; z1 ; z2; z3];
   else
   z_m = [z4; z1 ; z2];  
   end
   xhat = xhat + K * (z_m - zhat);
   P = (I - K * H) * P * (I - K * H)' + K * R * K';
   
   % UKF
   [root,p] = chol(n*Pukf);
   for i = 1 : n
       sigma(:,i) = xhatukf + root(i,:)';
       sigma(:,i+n) = xhatukf - root(i,:)';
   end
   for i = 1 : n2
       xbreve(:,i) = sigma(:,i);
   end

   for i = 1 : n2
   for tau = dt : dt : T/60
       tid = t_luc(t) + tau;
       [~, idx] = min(abs(time_luc - tid));
       Fin = Feeding_luc(idx)*0.00210;
       if Fin <= 8.7576e-40
           Fin = 0;
       end
       xbrevedot(1,1) = Fin;
       xbrevedot(2,1) = -(xbreve(2) ./xbreve(1)) .*Fin + mu_max .*(xbreve(3) ./(Ks + xbreve(3))) .*xbreve(2) - kd .*xbreve(2); %Biomass
       xbrevedot(3,1) = (Sin .*Fin) ./ xbreve(1) - mu_max .*(xbreve(3) ./(Ks + xbreve(3))) .*xbreve(2) ./Y_XS - (xbreve(3) ./ xbreve(1)) .*Fin; % Substrate
       xbrevedot(4,1) = -(Fin./xbreve(1)) .* xbreve(4) + (mu_max .*(xbreve(3) ./(Ks + xbreve(3))) .*xbreve(2))./Y_CO2X;% - kd .*xbreve(2); %CO2
       xbreve(:,i) = xbreve(:,i) + xbrevedot * dt;
     end
   end
  xhatukf = zeros(n,1);
  for i = 1 : n2
      xhatukf = xhatukf + W(i) * xbreve(:,i);
  end
  Pukf = zeros(n,n);
  for i = 1 : n2
      Pukf = Pukf + W(i) * (xbreve(:,i) - xhatukf) * (xbreve(:,i) - xhatukf)';
  end
  Pukf = Pukf + Q;
  
  for i = 1 : n2
      zukf(:,i) = xbreve(:,i);
  end
  zhat = zeros(n,1);
  
  for i = 1 : n2
      zhat = zhat + W(i) * zukf(:,i);
  end
  Py = zeros(n,n);
  Pxy = zeros(n,n);
  
  for i = 1 : n2
      Py = Py + W(i) * (zukf(:,i) - zhat(:)) * (zukf(:,i) - zhat(:))';
      Pxy = Pxy + W(i) * (xbreve(:,i) - xhatukf) * (zukf(:,i) - zhat(:))';
  end
  Py = H*Py*H' + R;
  Kukf = Pxy*H' * pinv(Py);
  xhatukf = xhatukf + Kukf * (z_m - H*zhat);
  Pukf = Pukf - Kukf * Py * Kukf';      
   
   % Data
   xArray(:,t) = x;
   xhatArray(:,t) = xhat;
   xhatukfArray(:,t) = xhatukf;
   PArray(:,t) = diag(P);
   PukfArray(:,t) = diag(Pukf);
   z1Array(t) = z1;
   z2Array(t) = z2;
   z3Array(t) = z3;
   toc
end 

%% Print your results

opts.Colors     = get(groot,'defaultAxesColorOrder');
opts.saveFolder = 'img/';
opts.width      = 8;
opts.height     = 6;
opts.fontType   = 'Times';
opts.fontSize   = 9;

% create new figure
fig = figure; clf

box on
hold on
plot(t_luc(1:tend), xArray(2,1:tend),'-.k','LineWidth',2)
plot(t_luc(1:tend), xhatukfArray(2,1:tend),'-','Color', [0.5 0.5 0.5],'LineWidth',2)
plot(t_luc(1:tend), xhatArray(2,1:tend),'--','Color', [0.5 0.5 0.5],'LineWidth',2)
time = t_CDW;
y = CDW;
err = std_CDW;
errorbar(time,y,err,'ob')

% add axis labes and legend
axis tight
xlabel('Time (h)')
ylabel('Biomass (g/L)')
legend('Model','UKF','EKF','off-line CDW','location', 'best')
title('(a)')

% remove unnecessary white space
set(gca,'LineWidth', 1.5,'LooseInset',max(get(gca,'TightInset'), 0.02))

% scaling
fig.Units               = 'centimeters';
fig.Position(3)         = opts.width;
fig.Position(4)         = opts.height;

% set text properties
set(fig.Children, ...
    'FontName',     'Times', ...
    'FontSize',     9);

% export to png
fig.PaperPositionMode   = 'auto';
saveas(fig,'Biomass.png');

%%

% create real measurement figure
fig = figure; clf

box on
hold on
plot(t_luc(1:tend), 0.6655*NIR(1:tend) + (1-0.6655) * CO2_Conc_Actual(1:tend),'-.k','LineWidth',2)
plot(t_luc(1:tend), xhatukfArray(2,1:tend),'-','Color', [0.5 0.5 0.5],'LineWidth',2)
time = t_CDW;
y = CDW;
err = std_CDW;
errorbar(time,y,err,'ob')

% add axis labes and legend
axis tight
xlabel('Time (h)')
ylabel('Biomass (g/L)')
legend('Sensor','UKF','off-line CDW','location', 'best')
title('(b)')

% remove unnecessary white space
set(gca,'LineWidth', 1.5,'LooseInset',max(get(gca,'TightInset'), 0.02))

% scaling
fig.Units               = 'centimeters';
fig.Position(3)         = opts.width;
fig.Position(4)         = opts.height;

% set text properties
set(fig.Children, ...
    'FontName',     'Times', ...
    'FontSize',     9);

% export to png
fig.PaperPositionMode   = 'auto';
saveas(fig,'NIR.png');

%%
% create Sugar figure
fig = figure; clf

box on
hold on
plot(t_luc(1:tend), xArray(3,1:tend),'-.k','LineWidth',2)
plot(t_luc(1:tend), xhatukfArray(3,1:tend),'-','Color', [0.5 0.5 0.5],'LineWidth',2)
plot(t_luc(1:tend), xhatArray(3,1:tend),'--','Color', [0.5 0.5 0.5],'LineWidth',2)
plot(t_S, Sugar,'ob')

% add axis labes and legend
axis tight
xlabel('Time (h)')
ylabel('Glucose (g/L)')
legend('Model','UKF','EKF','off-line S','location', 'best')
title('(a)')

% remove unnecessary white space
set(gca,'LineWidth', 1.5,'LooseInset',max(get(gca,'TightInset'), 0.02))

% scaling
fig.Units               = 'centimeters';
fig.Position(3)         = opts.width;
fig.Position(4)         = opts.height;

% set text properties
set(fig.Children, ...
    'FontName',     'Times', ...
    'FontSize',     9);

% export to png
fig.PaperPositionMode   = 'auto';
saveas(fig,'Sugar.png');

%%

% create Sugar real measurement figure
fig = figure; clf

box on
hold on
plot(t_luc(1:tend), z3Array(1:tend),'-.k','LineWidth',2)
plot(t_luc(1:tend), xhatukfArray(3,1:tend),'-','Color', [0.5 0.5 0.5],'LineWidth',2)
plot(t_S, Sugar,'ob')

% add axis labes and legend
axis tight
xlabel('Time (h)')
ylabel('Glucose (g/L)')
legend('Sensor','UKF','off-line S','location', 'best')
title('(b)')

% remove unnecessary white space
set(gca,'LineWidth', 1.5,'LooseInset',max(get(gca,'TightInset'), 0.02))

% scaling
fig.Units               = 'centimeters';
fig.Position(3)         = opts.width;
fig.Position(4)         = opts.height;

% set text properties
set(fig.Children, ...
    'FontName',     'Times', ...
    'FontSize',     9);

% export to png
fig.PaperPositionMode   = 'auto';
saveas(fig,'Glucose.png');

%% Volume Function
function [Time,Vol] = Volume_calc()

global tspan feed

y0 = [1.5];

[Time,Vol] = ode23tb(@(t,y) Volume(t,y), tspan, y0);

function [dV] = Volume(t,y)
    
Vol = y;
% Feed
[~, idx] = min(abs(tspan - t));
Feed = feed(idx)*0.00210;

dV = zeros(1,1);
dV(1) = Feed; %Volume
        
end
end