function  [dydt] = dynamics(t,y,tspan,rpmlist,feed,pH_val,CO2_conc,Air_flow,Temp)

% Variable Inputs
[~, idx] = min(abs(tspan - t));
rpm = rpmlist(idx);
u = feed(idx)*0.00210;
%pH = pH_val(idx);
pH = 7;
CO2_conc = CO2_conc(idx);
Air_flow = Air_flow(idx);
%Temp = Temp(idx)+273.15;
Temp = 30+273.15;

% Inputs
V = y(1); X = y(2); S = y(3); CO2 = y(4); HCO3 = y(5); CO2_d = y(6);

% % Parameter inputs
% Ks = 0.5646;         % g L^-1 
% Y_XS = 0.4116;
% mu_max = 0.1749;   % h^-1
% kd = 0.0011;
% Sin = 100;         % g/L
% 
% Y_CO2X = 0.5529; %estimated

% Parameter inputs
Ks = 0.5669;         % g L^-1 
Y_XS = 0.4520;
mu_max = 0.1696;   % h^-1
kd = 0.001;
Sin = 100;         % g/L

Y_CO2X = 0.7567; %estimated

%Y_XP =
R = 8.205736*10^-5; % (atm*m3)/(K*mol)
%Temp = 30+273.15; %K
%H = 0.0336; % (atm*m3)/(mol)

H = 101330*exp(11.549-(2440.4/(Temp))); %[Pa L /mol]
H = (H/101325)/1000; % (atm*m3)/(mol)

% klA calculation
rps = rpm/60; %rps
Qg = (Air_flow/(60*1000)); %m3/s
T = 0.115; %m
D = 0.046; %m
A = pi*(T/2)^2; %m2
N = rps;
Ncd = (4*(Qg^0.5)*(T^0.25))/(D^2); %rps
Vg = Qg/A; %m/s
%kLa_CO2 = (0.95*((N/Ncd)^1.54)*(Vg^1.27))*3600; %1/s -> 1/hr
kLa_CO2 = (1.59*((N/Ncd)^1.342)*((T/D)^0.415)*(Vg^0.93))*3600; %1/s -> 1/hr
%kLa_CO2 = (3.35*((N/Ncd)^1.19)*(Vg^0.549))*3600; %1/s -> 1/hr

%kLa_CO2 = 0.7669;

% CO2 dissociation

% This values are at 25 C from: 
% "Determination_of_CER_using_online_analysis_during_dynamic_biodegradation_experiments, 1996"
% k1 = 0.016 * 3600; %h-1
% k2 = 5000 * 3600; %L  mol-1  h-1
% K1 = 4.280 * 10^-7; % mol  L-1
% K2 = 4.280 * 10^+7; %L  mol-1

% values of k1 and k2 from: "THE COMBINATION OF CARBON DIOXIDE WITH
% HYDROXIDE IONS, 1956"
k1 = 0.036 * 3600; %h-1
k2 = 11600 * 3600; %L  mol-1  h-1

% Kw from: "Handbook of Biochemical Kinetics, 1999"
Kw = 10^-13.8; %at 30 C

% K1, K2, k_1 and k_2 from:
% "Determination_of_CER_using_online_analysis_during_dynamic_biodegradation_experiments"
K1 = exp(-11.582-918.9/Temp);
K2 = K1/Kw;

k_1 = k1/K1;
k_2 = k2/K2;

M_CO2 = 44; %g/mol
CO2_density = 1.8; % g/L

CO2_mol = ((Air_flow.*(CO2_conc)./100) .*CO2_density) ./(Air_flow .*M_CO2);

Delta_C = (CO2_mol/H)*R*Temp - CO2_d;

% Differential equations:
dV = u;
dX = -(X ./V) .*u + mu_max .*(S ./(Ks + S)) .*X - kd .*X; %Biomass
dS = (Sin .*u) ./ V - mu_max .*(S ./(Ks + S)) .*X ./Y_XS - (S ./ V) .*u; % Substrate
dCO2 = -(u./V) .* CO2 + (mu_max .*(S ./(Ks + S)) .*X)./Y_CO2X - kd .*X./Y_CO2X; %CO2
dHCO3 = (k1 + k2 * 10^(pH-14)) * CO2_d - (k_2 + k_1 * 10^(-pH)) * HCO3; %HCO3-
d_CO2_d = kLa_CO2 * Delta_C;

% Initial empty vector
dydt = zeros(6,1); % This creates an empty output column vector
% Output:
dydt(1)= dV; 
dydt(2)= dX; 
dydt(3)= dS;
dydt(4)= dCO2;
dydt(5)= dHCO3;
dydt(6)= d_CO2_d;

