function [y,t] = solver(t0,dt,tend,rpmlist,feed,pH,CO2_conc,Air_flow,Temper)
tspan = [t0:dt:tend];
% in the following the size of tspan is reduced from 18001 to 18000 to make
% the code run:
tspan = tspan(1:16886);
% Constants
R = 8.205736*10^-5; % (atm*m3)/(K*mol)
Temp = 30+273.15; %K
H = 0.0336; % (atm*m3)/(mol)
M_CO2 = 44; %g/mol
CO2_density = 1.8; % g/L
%calculate K2
% Kw from: "Handbook of Biochemical Kinetics, 1999"
Kw = 10^-13.8; %at 30 C
% K1, K2, k_1 and k_2 from:
% "Determination_of_CER_using_online_analysis_during_dynamic_biodegradation_experiments"
K1 = exp(-11.582-918.9/Temp);
K2 = K1/Kw;
Vol = 1.5; %L
Bio = 1.8; %g/L
Sub = 20; %g/L
CO2 = 0; %g/L
CO2_init = (((2*0.0413)/100)*CO2_density)/(2*M_CO2); %mol/L
CO2_d = R*Temp*(CO2_init/H); %mol/L
HCO3 = 0; %K2*(CO2_d)^2; %mol/L
% Solver initial conditions
y0 = [Vol Bio Sub CO2 HCO3 CO2_d];
%vec.Fin_exp = Fin_exp;
%odeopt = odeset('AbsTol',1e-2,'Reltol',1e-2);
[t,y]=ode15s(@(t,y) dynamics(t,y,tspan,rpmlist,feed,pH,CO2_conc,Air_flow,Temper),tspan,y0);
% Extract the information:
V = y(:,1);
X = y(:,2);
S = y(:,3); 
CO2 = y(:,4);
HCO3 = y(:,5);
CO2_d = y(:,6);
t=t;
end